/***************************************************************************************
Copyright (c) 2012 Michael Pozhidaev <msp@altlinux.org>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**************************************************************************************************/

#include "Solver.h"
#include"minisat.h"
#include<stdio.h>

using namespace Minisat;

extern "C" int minisat_solve(size_t varCount,
			     size_t clauseCount,
			     size_t* litCounts,
			     int* clauses[],
			     unsigned char* res,
			     unsigned char* collisions)
{
  if (litCounts == NULL || clauses == NULL || res == NULL || collisions == NULL)
    return MINISAT_INVALID_DATA;
  if (varCount == 0)
    return MINISAT_INVALID_DATA;
  Solver S;
  for(size_t i = 0;i < clauseCount;i++)
    {
      if (clauses[i] == NULL)
	return MINISAT_INVALID_DATA;
      vec<Lit> lits;
      for(size_t k = 0;k < litCounts[i];k++)
	{
	  const int var = abs(clauses[i][k]) - 1;
	  if (var >= varCount)
	    return MINISAT_INVALID_DATA;
	  while (var >= S.nVars()) 
	    S.newVar();
	  lits.push(clauses[i][k] > 0 ? mkLit(var) : ~mkLit(var));
	}
      if (!S.addClause(lits))
	{
	  for(size_t k = 1;k <= varCount;k++)
	    collisions[k] = 0;
	  for(size_t k = 0;k < litCounts[i];k++)
	    collisions[abs(clauses[i][k])] = 1;
	  return MINISAT_UNSAT;
	}
    }
  /*Not clear right now is this really needed;
  if (!S.simplify())
    return MINISAT_UNSAT;
  */
  if (!S.solve())
    {
      for(size_t i = 1;i <= varCount;i++)
	collisions[i] = 0;
      for(size_t i = 0;i < S.conflict.size();i++)
	collisions[var(S.conflict[i]) + 1] = 1;
      return MINISAT_UNSAT;
    }
  for (int i = 0; i < S.nVars(); i++)
    {
      if (S.model[i] == l_Undef)
	return MINISAT_INTERNAL_PROBLEM;
      res[i + 1] = S.model[i] == l_True;
    }
  return MINISAT_OK;
}
